#ifndef __JSA_HPP__
#define __JSA_HPP__

#ifdef WINJSA_DLL
#define WINJSA_EXPORT __declspec(dllexport)
#else
#define WINJSA_EXPORT __declspec(dllimport)
#endif

namespace JSA
{
    struct MatchData
    {
        enum MatchCodes
        {
            LZLiteral = 1,
            LZMatch = 2,
            LZDone = 3,
            LZError = 0xFFFFFFFF
        };
        unsigned long LZMatchData;
        unsigned long long DataPosition;
        unsigned long long LZWindowIndex;
        unsigned long LZMatchLength;
        const unsigned char * LZMatchBytes;
        unsigned char LZLiteralByte;
    };

    struct DictionaryOptions
    {
        unsigned long long WindowSize;
        unsigned long MinMatchLength;
        unsigned long MaxMatchLength;
        unsigned long ChainLimit;
    };

    class WINJSA_EXPORT IDictionary
    {
    public:
        virtual ~IDictionary();

        virtual bool GetDictionaryOptions(DictionaryOptions & rOptions) const = 0;
        virtual bool FindMatch(MatchData & rMatchData) const = 0;
        virtual bool Update(const MatchData & rMatchData) = 0;
        virtual bool Finalize() = 0; // this should be called when psMatchData->LZMatchData is MatchData::LZDone
        virtual unsigned long long GetDataLength() const = 0;
        virtual unsigned long long GetDataPosition() const = 0;

        static IDictionary * GetDefaultDictionary(const DictionaryOptions & udtOptions,
                                                  const unsigned char * pData,unsigned long long nDataLength); // min match length must be atleast 2
    };

    class WINJSA_EXPORT ICompressor
    {
    public:
        virtual ~ICompressor();

        virtual bool Encode(const MatchData & rMatchData) = 0;
        virtual bool Update() = 0;
        virtual bool Finalize() = 0;
    };

    class WINJSA_EXPORT IDecompressor
    {
    public:
        virtual ~IDecompressor();

        virtual bool Decode() = 0;
        virtual bool Update() = 0;
        virtual bool Finalize() = 0;
    };
}

#endif // __JSA_HPP__
