#ifndef JSAPRIVATE_HPP_INCLUDED
#define JSAPRIVATE_HPP_INCLUDED

#include "JSA.hpp"

namespace JSA
{
    class CHashList;

    struct HashNode
    {
        CHashList * OwnerList;
        unsigned long long Position;
        const unsigned char * DataPtr;

        HashNode * PrevNode;
        HashNode * NextNode;
    };

    class CHashList
    {
    public:
        CHashList();
        virtual ~CHashList();

        void Add(HashNode * pNode,unsigned long long nChainLimit);
        void Remove(HashNode * pNode);
        inline unsigned long long GetNodeCount() const { return m_nNodeCount; }

        inline const HashNode * GetFirstHash() const { return m_pFirstHash; }
        inline HashNode * GetFirstHash() { return m_pFirstHash; }

    protected:
        HashNode * m_pFirstHash;
        HashNode * m_pLastHash;
        unsigned long long m_nNodeCount;
    };

    class HashNodeAllocator
    {
    public:
        HashNodeAllocator(const DictionaryOptions & dwOptions);
        virtual ~HashNodeAllocator();

        HashNode * GetNode();
        inline unsigned long long GetNodeCount() const { return m_nNodeCount; }

    protected:
        HashNode * m_pHashNodes;
        unsigned long long m_nNodeCount;
        unsigned long long m_nCurrentNode;
    };

    class CHashTable
    {
    public:
        enum DefaultValues
        {
            TwoBytes = 65536
        };

        CHashTable();
        virtual ~CHashTable();

        virtual const CHashList & GetHashList(unsigned long long nHashCode) const;
        virtual CHashList & GetHashList(unsigned long long nHashCode);

    protected:
        CHashList * m_pHashLists;
        unsigned long long m_nListCount;
    };

    class CDictionary : public IDictionary
    {
    public:
        CDictionary(const DictionaryOptions & options,const unsigned char * pData,unsigned long long nDataLength);
        virtual ~CDictionary();

        virtual bool GetDictionaryOptions(DictionaryOptions & rOptions) const;
        virtual bool FindMatch(MatchData & rMatchData) const;
        virtual bool Update(const MatchData & rMatchData);
        virtual bool Finalize(); // this should be called when pMatchData->LZMatchData is MatchData::LZDone
        virtual unsigned long long GetDataLength() const;
        virtual unsigned long long GetDataPosition() const;

    protected:
        DictionaryOptions m_udtOptions;
        const unsigned char * m_pData;
        unsigned long long m_nDataLength;

        unsigned long long m_nDictPos;
        unsigned long long m_nWindowLength;
        unsigned long long m_nOldestPos;

        HashNodeAllocator * m_pHashNodeAllocator;
        CHashTable * m_pHashTable;

        static unsigned long long CompareData(const unsigned char * pDictData,const unsigned char * pData,unsigned long long nPreMatchedLength,
                                              unsigned long long nMaxMatchLength);
    };
}

#endif
